package io.hmit.modules.committee.controller;

import io.hmit.common.annotation.LogOperation;
import io.hmit.common.constant.Constant;
import io.hmit.common.page.PageData;
import io.hmit.common.utils.ExcelUtils;
import io.hmit.common.utils.Result;
import io.hmit.common.validator.AssertUtils;
import io.hmit.common.validator.ValidatorUtils;
import io.hmit.common.validator.group.AddGroup;
import io.hmit.common.validator.group.DefaultGroup;
import io.hmit.common.validator.group.UpdateGroup;
import io.hmit.modules.area.dto.AreaDTO;
import io.hmit.modules.committee.dto.CommitteeDTO;
import io.hmit.modules.committee.excel.CommitteeExcel;
import io.hmit.modules.committee.service.CommitteeService;
import io.hmit.modules.user.dto.UserDTO;
import io.hmit.modules.user.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * 委员会管理
 *
 * @author syf syf0412@qq.com
 * @since 1.0.0 2020-12-25
 */
@RestController
@RequestMapping("committee/committee")
@Api(tags="委员会管理")
public class CommitteeController {
    @Autowired
    private CommitteeService committeeService;

    @Autowired
    private UserService userService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataType="String") ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataType="String")
    })
//    @RequiresPermissions("committee:committee:page")
    public Result<PageData<CommitteeDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<CommitteeDTO> page = committeeService.page(params);

        return new Result<PageData<CommitteeDTO>>().ok(page);
    }

    @GetMapping("list")
    @ApiOperation("列表")
    public Result<List<CommitteeDTO>> list(@ApiIgnore @RequestParam Map<String, Object> params) {

        List<CommitteeDTO> list = committeeService.list(params);

        return new Result<List<CommitteeDTO>>().ok(list);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
//    @RequiresPermissions("committee:committee:info")
    public Result<CommitteeDTO> get(@PathVariable("id") Long id){
        CommitteeDTO data = committeeService.get(id);

        return new Result<CommitteeDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
//    @RequiresPermissions("committee:committee:save")
    public Result save(@RequestBody CommitteeDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        committeeService.save(dto);

        return new Result();
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
    //@RequiresPermissions("committee:committee:update")
    public Result update(@RequestBody CommitteeDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        committeeService.update(dto);

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
    //@RequiresPermissions("committee:committee:delete")
    public Result delete(@RequestBody Long[] ids){
        //校验数据
        AssertUtils.isArrayEmpty(ids, "id");
        for (Long id : ids) {
            Map<String, Object> params = new HashMap<>();
            params.put("committee",id);
            List<UserDTO> userList = userService.list(params);
            if(userList.size()>0){
                return new Result().error(500,"委员会 "+committeeService.get(id).getName()+" 存在会员信息，请核对后再进行删除");
            }
            committeeService.deleteById(id);
        }
//        committeeService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
    //@RequiresPermissions("committee:committee:export")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<CommitteeDTO> list = committeeService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, CommitteeExcel.class);
    }

}
