package io.hmit.modules.meeting.controller;

import io.hmit.common.annotation.LogOperation;
import io.hmit.common.constant.Constant;
import io.hmit.common.page.PageData;
import io.hmit.common.utils.ExcelUtils;
import io.hmit.common.utils.Result;
import io.hmit.common.validator.AssertUtils;
import io.hmit.common.validator.ValidatorUtils;
import io.hmit.common.validator.group.AddGroup;
import io.hmit.common.validator.group.DefaultGroup;
import io.hmit.common.validator.group.UpdateGroup;
import io.hmit.modules.meeting.dto.MeetingDTO;
import io.hmit.modules.meeting.dto.WlMeetingStaffDTO;
import io.hmit.modules.meeting.excel.MeetingExcel;
import io.hmit.modules.meeting.service.MeetingService;
import io.hmit.modules.meeting.service.WlMeetingStaffService;
import io.hmit.modules.user.dto.UserDTO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * 会议管理
 *
 * @author syf syf0412@qq.com
 * @since 1.0.0 2020-12-25
 */
@RestController
@RequestMapping("meeting/meeting")
@Api(tags="会议管理")
public class MeetingController {
    @Autowired
    private MeetingService meetingService;

    @Autowired
    private WlMeetingStaffService wlMeetingStaffService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataType="String") ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataType="String"),
        @ApiImplicitParam(name = "name", value = "会议名称", paramType = "query", dataType="String")
    })
     //@RequiresPermissions("meeting:meeting:page")
    public Result<PageData<MeetingDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<MeetingDTO> page = meetingService.page(params);

        return new Result<PageData<MeetingDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
     //@RequiresPermissions("meeting:meeting:info")
    public Result<MeetingDTO> get(@PathVariable("id") Long id){
        MeetingDTO data = meetingService.get(id);

        List<UserDTO> userNames = wlMeetingStaffService.getInvitedNames(id);
        data.setInvitedUserNames(userNames);

        return new Result<MeetingDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
     //@RequiresPermissions("meeting:meeting:save")
    public Result save(@RequestBody MeetingDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        meetingService.save(dto);

        return new Result();
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
     //@RequiresPermissions("meeting:meeting:update")
    public Result update(@RequestBody MeetingDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        meetingService.update(dto);

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
     //@RequiresPermissions("meeting:meeting:delete")
    public Result delete(@RequestBody Long[] ids){
        //校验数据
        AssertUtils.isArrayEmpty(ids, "id");

        for (Long id : ids) {
            Map<String, Object> params = new HashMap<>();
            params.put("meetingId",id);
            List<WlMeetingStaffDTO> meetingList = wlMeetingStaffService.list(params);
            if(meetingList.size()>0){
                return new Result().error(500,"会议 "+meetingService.get(id).getTitle()+" 存在会员信息，请核对后再进行删除");
            }
            meetingService.deleteById(id);
        }
//        meetingService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
     //@RequiresPermissions("meeting:meeting:export")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<MeetingDTO> list = meetingService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, MeetingExcel.class);
    }

}
