package io.hmit.modules.meeting.controller;

import io.hmit.common.annotation.LogOperation;
import io.hmit.common.constant.Constant;
import io.hmit.common.page.PageData;
import io.hmit.common.utils.ExcelUtils;
import io.hmit.common.utils.Result;
import io.hmit.common.validator.AssertUtils;
import io.hmit.common.validator.ValidatorUtils;
import io.hmit.common.validator.group.AddGroup;
import io.hmit.common.validator.group.DefaultGroup;
import io.hmit.common.validator.group.UpdateGroup;
import io.hmit.modules.area.service.AreaService;
import io.hmit.modules.committee.service.CommitteeService;
import io.hmit.modules.meeting.dto.WlMeetingStaffDTO;
import io.hmit.modules.meeting.excel.WlMeetingStaffExcel;
import io.hmit.modules.meeting.service.WlMeetingStaffService;
import io.hmit.modules.user.dto.UserDTO;
import io.hmit.modules.user.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;


/**
 * 参会人员
 *
 * @author zsh 408538940@qq.com
 * @since 1.0.0 2020-12-29
 */
@RestController
@RequestMapping("meeting/wlmeetingstaff")
@Api(tags="参会人员")
public class WlMeetingStaffController {
    @Autowired
    private WlMeetingStaffService wlMeetingStaffService;

    @Autowired
    private UserService userService;

    @Autowired
    private CommitteeService committeeService;

    @Autowired
    private AreaService areaService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataType="String") ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataType="String"),
        @ApiImplicitParam(name = "meetingId", value = "会议ID", paramType = "query", dataType="long"),
        @ApiImplicitParam(name = "registrationState", value = "是否报名 1：已报名 2：未报名", paramType = "query", dataType="int")
    })
//    @RequiresPermissions("meeting:wlmeetingstaff:page")
    public Result<PageData<WlMeetingStaffDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<WlMeetingStaffDTO> page = wlMeetingStaffService.page(params);

        page.getList().forEach(wlMeetingStaff -> {
            UserDTO userDTO= userService.get(wlMeetingStaff.getUserId());
            if(userDTO.getCommitteeId()!= null && userDTO.getAreaId() != null){
                userDTO.setCommitteeName(committeeService.get(userDTO.getCommitteeId()).getName());
                userDTO.setAreaName(areaService.get(userDTO.getAreaId()).getName());
                wlMeetingStaff.setUserInfo(userDTO);
                return;
            }
            userDTO.setCommitteeName("暂无所属委员会");
            userDTO.setAreaName("暂无所属地区");
            wlMeetingStaff.setUserInfo(userDTO);
        });
        return new Result<PageData<WlMeetingStaffDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
//    @RequiresPermissions("meeting:wlmeetingstaff:info")
    public Result<WlMeetingStaffDTO> get(@PathVariable("id") Long id){
        WlMeetingStaffDTO data = wlMeetingStaffService.get(id);

        return new Result<WlMeetingStaffDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
//    @RequiresPermissions("meeting:wlmeetingstaff:save")
    public Result save(@RequestBody WlMeetingStaffDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        wlMeetingStaffService.save(dto);

        return new Result();
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
//    @RequiresPermissions("meeting:wlmeetingstaff:update")
    public Result update(@RequestBody WlMeetingStaffDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        wlMeetingStaffService.update(dto);

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
//    @RequiresPermissions("meeting:wlmeetingstaff:delete")
    public Result delete(@RequestBody Long[] ids){
        //校验数据
        AssertUtils.isArrayEmpty(ids, "id");

        wlMeetingStaffService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
//    @RequiresPermissions("meeting:wlmeetingstaff:export")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<WlMeetingStaffDTO> list = wlMeetingStaffService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, WlMeetingStaffExcel.class);
    }

}
