package io.hmit.modules.user.controller;

import io.hmit.common.annotation.LogOperation;
import io.hmit.common.constant.Constant;
import io.hmit.common.page.PageData;
import io.hmit.common.utils.ExcelUtils;
import io.hmit.common.utils.Result;
import io.hmit.common.validator.AssertUtils;
import io.hmit.common.validator.ValidatorUtils;
import io.hmit.common.validator.group.AddGroup;
import io.hmit.common.validator.group.DefaultGroup;
import io.hmit.common.validator.group.UpdateGroup;
import io.hmit.modules.area.dto.AreaDTO;
import io.hmit.modules.area.service.AreaService;
import io.hmit.modules.committee.service.CommitteeService;
import io.hmit.modules.user.dto.AreaUserDTO;
import io.hmit.modules.user.dto.CommitteeUserDTO;
import io.hmit.modules.user.dto.UserDTO;
import io.hmit.modules.user.excel.UserExcel;
import io.hmit.modules.user.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * 用户信息表（关注公众号的用户）
 *
 * @author syf syf0412@qq.com
 * @since 1.0.0 2020-12-25
 */
@RestController
@RequestMapping("user/user")
@Api(tags="用户信息表（关注公众号的用户）")
public class UserController {
    @Autowired
    private UserService userService;

    @Autowired
    private CommitteeService committeeService;

    @Autowired
    private AreaService areaService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataType="String") ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataType="String"),
        @ApiImplicitParam(name = "check", value = "是否审核：0,待审核（审核用户列表）", paramType = "query", dataType="int"),
        @ApiImplicitParam(name = "company", value = "单位名称", paramType = "query", dataType="String"),
        @ApiImplicitParam(name = "area", value = "区市ID", paramType = "query", dataType="long"),
        @ApiImplicitParam(name = "mobile", value = "手机号码", paramType = "query", dataType="String"),
        @ApiImplicitParam(name = "name", value = "姓名", paramType = "query", dataType="String"),
        @ApiImplicitParam(name = "committee", value = "委员会ID", paramType = "query", dataType="long")
    })
     //@RequiresPermissions("user:user:page")
    public Result<PageData<UserDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<UserDTO> page = userService.page(params);
        page.getList().forEach(userDTO -> {
            if(userDTO.getCommitteeId()!= null && userDTO.getAreaId() != null){
                userDTO.setCommitteeName(committeeService.get(userDTO.getCommitteeId()).getName());
                userDTO.setAreaName(areaService.get(userDTO.getAreaId()).getName());
                return;
            }
            userDTO.setCommitteeName("暂无所属委员会");
            userDTO.setAreaName("暂无所属地区");
        });

        return new Result<PageData<UserDTO>>().ok(page);
    }
    @GetMapping("committeeList")
    @ApiOperation("根据委员会获取用户信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "committee", value = "委员会Id", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "name", value = "人员姓名", paramType = "query", dataType = "String")
    })
    public Result<List<CommitteeUserDTO>> CommitteeUserList(@ApiIgnore @RequestParam Map<String, Object> params) {

        List<CommitteeUserDTO> list = userService.getCommitteeUserList(params);

        return new Result<List<CommitteeUserDTO>>().ok(list);
    }

    @GetMapping("areaList")
    @ApiOperation("根据区市获取用户信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "area", value = "区市Id", paramType = "query",dataType = "long"),
            @ApiImplicitParam(name = "name", value = "人员姓名", paramType = "query", dataType = "String")
    })
    public Result<List<AreaUserDTO>> AreaUserList(@ApiIgnore @RequestParam Map<String, Object> params) {

        List<AreaUserDTO> list = userService.getAreaUserList(params);

        return new Result<List<AreaUserDTO>>().ok(list);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
     //@RequiresPermissions("user:user:info")
    public Result<UserDTO> get(@PathVariable("id") Long id){
        UserDTO data = userService.get(id);

        if(data.getCommitteeId()!= null && data.getAreaId() != null){
            data.setCommitteeName(committeeService.get(data.getCommitteeId()).getName());
            data.setAreaName(areaService.get(data.getAreaId()).getName());
        }else {
            data.setAreaName("暂无所属地区");
            data.setCommitteeName("暂无所属委员会");
        }

        return new Result<UserDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
     //@RequiresPermissions("user:user:save")
    public Result save(@RequestBody UserDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        userService.save(dto);

        return new Result();
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
     //@RequiresPermissions("user:user:update")
    public Result update(@RequestBody UserDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        userService.update(dto);

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
     //@RequiresPermissions("user:user:delete")
    public Result delete(@RequestBody Long[] ids){
        //校验数据
        AssertUtils.isArrayEmpty(ids, "id");

        userService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
     //@RequiresPermissions("user:user:export")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<UserDTO> list = userService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, UserExcel.class);
    }

}
